@@ -62,8 +62,9 @@ as_sparta.data.frame <- function(x) {
   if (!all(lapply(x, class) == "character")) {
      stop("all varibles must be of class 'character'", call. = FALSE)
   }
-  # TODO: consider aggregate(U ~ A + S + T + L, x, sum)
-  # where U = 1L
+  # aggregate(rep(1L, nrow(x)) ~ gear + carb, FUN = sum, data = x)
+  # aggregate(make_form(colnames(derma)[1:8]), data = derma[, 1:8], FUN = sum)
+  # TODO: TRY IT OUT FOR LARGE DATASETS! JUST rbind derma a couple of times
   as_sparta(table(x, dnn = colnames(x)))
 }
 
@@ -101,6 +102,13 @@ as_array.sparta <- function(x) {
   arr
 }
 
+#' @rdname as_array
+#' @export
+as_array.sparta_unity <- function(x) {
+  dim_ <- .map_int(attr(x, "dim_names"), function(z) length(z))
+  array(sparta_rank(x), dim = dim_, dimnames = attr(x, "dim_names"))
+}
+
 
 #' As data frame
 #'
